import EventEmitter from 'events'
import { PulseAudio,  percentToVolume } from 'pulseaudio.js'

export const paBus = (devices) => {
  const paBus = new EventEmitter()
  const pa = new PulseAudio();

  const initialize = (pa) => {
    paBus.promise = pa.connect()
      .then(() => console.info('PulseAudio connected'))
      .then(() => pa.getAllSinks())
      .then((sinks) => {
        const devicesSet = new Set(devices)
        sinks.forEach(({ index, name }) => {
          // Check by deleteing it
          if (devicesSet.delete(name)) {
            paBus.emit('deviceUpdate', { name, index })
          }
        })
      })
      .then(() => {
        pa.on('error', (e) => {
          console.warn('PusleAudio errored', e)
          paBus.emit('error', e)
          // Reboot
          initialize(pa)
        })
      })
      .catch((e) => {
        console.warn('Failed initializing PulseAudio', e)
        paBus.emit('error', e)
      })
  }

  initialize(pa)

  paBus.setSinkVolume = (value, device) => pa.setSinkVolume(percentToVolume(value * 100), device)
  paBus.moveSinkInput = (sinkInputId, device) => pa.moveSinkInput(sinkInputId, device)
  paBus.getSinkInputList = () => pa.getSinkInputList()

  return paBus
}
